/******************************************************************************
*
*       Port of Philips LPC901 Motor Control Program to PIC
*
*******************************************************************************

Hardware Interface: (See Protel 99SE schematic in 06117_MOTOR_CTL)

Controller board: PCM3 @ 16MHZ

TRIAC DRIVE:                RC0, RC1, and RC2
60HZ Interrupt Input:       RB0
Speed Increase Button:      RB4
Speed decrease button:      RB5
Harmonic reduction disable: RB3

RS232:                      RC6/RC7 (Caution: AC Line Connection to PIC)

*/
#define 18F452

#include "18F452.h"     // Device header file


#device ADC=10 *=16
#include <string.h>
#include <stdlib.h>
#include <stdio.h>


#use fast_io(b)
#use fast_io(c)
#use fast_io(d)

#use delay(clock=16000000)

#use rs232(baud=57600, xmit=PIN_C6, rcv=PIN_C7)

#fuses HS, NOPROTECT, PUT, BROWNOUT, NOLVP, WDT128

#priority EXT, TIMER1, TIMER0


/******************************************************************************/
/*PIC18F4520/452 only configuration register addresses                             */
/******************************************************************************/

#byte   PORTA   =   0xF80       // Define the address of the I/O
#byte   PORTB   =   0xF81       // Define the address of the I/O
#byte   PORTC   =   0xF82       // ports for direct reading and writing
#byte   PORTD   =   0xF83       // ports for direct reading and writing


#byte   TMR1L   = 0xFCE         //Timer 1 count register low byte
#byte   TMR1H   = 0xFCF         //Timer 1 count register high byte
#byte   CCPR1L  = 0xFBE         //PWM1 register 1
#byte   CCPR1H  = 0xFBF         //PWM1 register 2
#byte   CCP1CON = 0xFBD         //CCP1 control register
#byte   ADCON0  = 0xFC2         //ADC control register 0
#byte   ADCON1  = 0xFC1         //ADC control register 1
#byte   ADCON2  = 0xFC0         //ADC control register 2
#byte   ADRESH  = 0xFC4         //ADC result high byte register
#byte   ADRESL  = 0xFC3         //ADC result low byte register
#byte   RCSTA   = 0xFAB         //Defines Receive status port

#byte   LATC    = 0xF8B

#bit    CREN    =   0xFAB.4     //Clear errors bit

#byte   PIR1   =    0xF9E
#byte   INTCON =    0xFF2
#byte   INTCON2 =   0xFF1
#byte   INTCON3 =   0xFF0
#byte   T0CON =     0xFD5
#byte   T1CON =     0xFCD
#byte   T2CON =     0xFCA
#byte   T3CON =     0xFB1


long    TIMER0;

#byte TIMER0 =0xFD6
#byte TMR0H =       0xFD7
#byte TMR0L =       0xFD6

#bit    INTCON_TMR0IF      =   0xFF2.2
#bit    INTCON_INT0IF      =   0xFF2.1
#bit    INTCON_GIE         =   0xFF2.7

#bit    PIR1_TMR1IF        =   0xF9E.0

long adc_value_copied;

int ac_interrupts=0;
//int old_ac_interrupts=0;

//short AC_FLAG = FALSE;


/******************************************************************************/
/*PIC A/D configuration register setup values (Pic Manual Page 111-112)       */
/******************************************************************************/

#define AD_SETUP_0  0b10000001  //ADCON0 setup byte (Fosc/32)
#define AD_SETUP_1  0b10000000  //ADCON1 setup byte (VREF=INT, 8 Analogs)
#bit    GO_DONE =   0xFC2.2     //ADC start conversion bit

/******************************************************************************/
/*IO port direction settings                                                  */
/******************************************************************************/

#define PORT_A_MASK 0b00011111
#define PORT_E_MASK 0b00000111

#define PORT_B_MASK 0b11111111      // Port B direction register mask
#define PORT_C_MASK 0b10000000      // Port C direction register mask
#define PORT_D_MASK 0b00000000      // Port D direction register mask

/******************************************************************************/
/*IO pin definitions                                                          */
/******************************************************************************/

// *********************** Port A ***********************************

#define AN0_INP         PIN_A0  //  1
#define AN1_INP         PIN_A1  //  1
#define AN2_INP         PIN_A2  //  1
#define VREF            PIN_A3  //  1
#define FREE1           PIN_A4  //  1
#define AN5_INP         PIN_E0  //  1
#define AN6_INP         PIN_E1  //  1
#define AN7_INP         PIN_E2  //  1

// *********************** Port B *************************************

#define ND1             PIN_B0  //  1 - 60 HZ Synchronization Trigger Input (ISR)
#define ND2             PIN_B1  //  0
#define ND3             PIN_B2  //  0
#define KEY3_IN         PIN_B3  //  1 - Harmonic Reduction disable
#define KEY1_IN         PIN_B4  //  1 - Speed Increase
#define KEY2_IN         PIN_B5  //  1 - Speed Decrease
#define ND7             PIN_B6  //  0
#define ND8             PIN_B7  //  0

//************************ Port C *************************************

#define ND9             PIN_C0  //  0 TRIAC GATE (active low w.r.t. VCC)
#define ND10            PIN_C1  //  0 TRIAC GATE
#define ND11            PIN_C2  //  0 TRIAC GATE
#define ND12            PIN_C3  //  0
#define ND13            PIN_C4  //  0
#define ND14            PIN_C5  //  0
#define TXD             PIN_C6  //  0 RS232 OUT
#define RXD             PIN_C7  //  1 RS232 INP

//************************ Port D *************************************

#define ND17            PIN_D0  //  0
#define ND18            PIN_D1  //  0
#define ND19            PIN_D2  //  0
#define ND20            PIN_D3  //  0
#define ND21            PIN_D4  //  0
#define ND22            PIN_D5  //  0
#define ND23            PIN_D6  //  0
#define ND24            PIN_D7  //  0


//long _compare_counter=0;


// prototypes





