

//***************************************************************************
//* Name: init()
//* Input(s) : none.
//* Returns : none.
//* Description : initialization of P89LPC901/PIC18F452
//***************************************************************************
void init(void)
{
	// Initialization of globals as declared in LPC901 application
	curr_flag = 0;
	syn = 0;
	PHASE = 0;
	adc_old = 0x0;
	one_step = 1;
	harm_flag = 0;
	update = 0;
	key1_flag = 0;
	key2_flag = 0;
	key3_flag = 0;

	key_value = 0x0;
	speed_max = 46; //from 0~140 degree, set as 46 levels, so 3 degree a level
	//key_value = speed_max;

	// Initialize port pins
	PORTA = 0b00000000;
	PORTB = 0b00000000;
	PORTC = 0b11000111;
	PORTD = 0b00000111;

	// Setup IO direction registers as defined
	set_tris_a(PORT_A_MASK);
	set_tris_b(PORT_B_MASK);
	set_tris_c(PORT_C_MASK);
	set_tris_d(PORT_D_MASK);

	//port_b_pullups(0xFF);

	//ADCON0 = AD_SETUP_0;                          // Setup ADC (SCU_R??.H)
	//ADCON1 = AD_SETUP_1;                          // Setup ADC ports

	// Set to 256khz drive to TMR0
	setup_timer_0(RTCC_INTERNAL | RTCC_DIV_256 | RTCC_8_BIT);
	setup_timer_1(T1_INTERNAL | T1_DIV_BY_1);       // Clock = 4MHZ, OF=16.384ms

	set_uart_speed(57600);

	// Test if we have a AC signal
	if(test_AC() == 1)
	{
		printf("\r\nNO LINE AC SIGNAL DETECTED\r\n");
	}

	set_timer1(0);
	set_timer0(0);

	printf("\r\nRestart value: %u\r\n", restart_cause());

	//Enable interrupts
	EXT_INT_EDGE(H_TO_L);
	enable_interrupts(INT_EXT);     // Power Line Interrupt at 50 or 60 HZ
	enable_interrupts(INT_TIMER0);  // Keys scanning interrupt at 16.384ms rate
	enable_interrupts(INT_TIMER1);  // TRIAC Timing

}



int test_AC()
{
	int new_B0=0, i, ac_flag = 1;

	// Test if we have a AC signal
	for(i=0;i<100;i++)
	{
		if(input(pin_B0) == 0)
			new_B0++;

		if(new_B0 > 50){
			ac_flag = 0;
			break;
		}
		delay_ms(10);
	}
	return(ac_flag);
}

//***************************************************************************
//* Name: startupdelay()
//* Input(s) : unsigned int degree, unsigned int cnt.
//* Returns : none.
//* Description : provide softstart function
//***************************************************************************
void startupdelay (unsigned long degree, unsigned long cnt)
{
    unsigned long i,j;
    for(i=170;i > degree;i--)
    {
        for(j=0;j<cnt;j++)
        {
			while(!syn);
			syn = 0;
        }
        PHASE = phase2timer(i);
        printf("PHASE: %lu %lu %lu\r\n", PHASE, i, j );
        restart_wdt();
    }
}



//***************************************************************************
//* Name: T1_ISR()
//* Input(s) : none.
//* Returns : none.
//* Description : Interrupt from Timer 1
//***************************************************************************
#int_timer1 NOCLEAR
void T1_ISR(void) //T0 interrupt vector address is 000Bh
{
	disable_interrupts(GLOBAL);
    while(PIR1_TMR1IF != 0x0)
    {
	    PIR1_TMR1IF = 0x0;
    }
    //fire TRIAC

    if(PHASE != 0x0000)
    {
        fire_triac();
    }
    disable_interrupts(INT_TIMER1);
    enable_interrupts(INT_EXT);
    enable_interrupts(GLOBAL);
}



//***************************************************************************

//* Name: KBI_ISR()
//* Input(s) : none.
//* Returns : none.
//* Description : Power Line ISR re-written for use with PIC (60HZ)
//* Is called at a 120 Hz rate due to INT edge being changed after each INT
//* event.
//***************************************************************************
#INT_EXT NOCLEAR
void KBI_ISR(void)
{
    output_high(PIN_D4);
    while(INTCON_INT0IF != 0x0)
    {
	    INTCON_INT0IF = 0;  		// Clear EXT INT bit
    }
    //Inverse the interrupt edge and Load Timer0 Phase Value
 	set_timer1(PHASE);

    if (curr_flag == 0) 			//falling edge caused interrupt
    {
        curr_flag = 1;
        EXT_INT_EDGE(L_TO_H);       // Set ext int on rising edge
    }
    else if (curr_flag != 0) 		//rising edge caused interrupt
    {
        curr_flag = 0;
        EXT_INT_EDGE(H_TO_L);       // Set ext int on falling edge
    }
    //harmonic reducing flag
    harm_flag++;
    if (harm_flag > 3)
    {
        output_high(PIN_D5);
        harm_flag = 0;

    }
    //soft start control
    if (harm_flag == 3)
    {
        update++;
        if (update > update_rate)
        {
            update = 0;
        }
    }
  	syn = 1;
    enable_interrupts(INT_TIMER1);
    disable_interrupts(INT_EXT);
    enable_interrupts(GLOBAL);
    output_low(PIN_D4);
    output_low(PIN_D5);
	ac_interrupts++;
}


//***************************************************************************
//* Name: T2_ISR()
//* Input(s) : none.
//* Returns : none.
//* Description : Interrupt from Timer 0 at 16.384 Khz rate
//***************************************************************************
#int_timer0
void T0_ISR(void)
{
	/* key1 sampling */
	if ((input(KEY1_IN) == 0) && (key1_flag == 0))
	{
		key1_flag = 1;
	}
	if ((input(KEY1_IN) == 0) && (key1_flag == 1))
	{
		key1_flag = 2;
	}
	if ((input(KEY1_IN) == 1) && (key1_flag == 2))
	{
		key1_flag = 3;
	}
	/* key2 sampling */
	if ((input(KEY2_IN) == 0) && (key2_flag == 0))
	{
		key2_flag = 1;
	}
	if ((input(KEY2_IN) == 0) && (key2_flag == 1))
	{
		key2_flag = 2;
	}
	if ((input(KEY2_IN) == 1) && (key2_flag == 2))
	{
		key2_flag = 3;
	}

	if (input(KEY3_IN) == 0) // Sense harmonic disable input pin
	{
		DISABLE_HARM_RED_FLAG = TRUE;
	}else{
		DISABLE_HARM_RED_FLAG = FALSE;
	}
}


//***************************************************************************
//* Name: fire_triac()
//* Input(s) : none.
//* Returns : none.
//* Description : fire TRIAC with P1.2, P3.0 and P3.1 together
//***************************************************************************
void fire_triac(void)
{
    PORTC &= 0xF8;	// Turn on TRIAC drive
    if (PHASE > 0xF000)
    {
        delay_us(400);//long fire pulse delay
    }
    else
    {
        delay_us(200);//short fire pulse delay
    }
    PORTC |= 0x07;	// Turn off TRIAC drive
}
//***************************************************************************
//* Name: get_speed()
//* Input(s) : none.
//* Returns : unsigned int phase_value.
//* Description : control routine in geting adc value, process softstart and look
//* up phase value according to the adc value.
//***************************************************************************
unsigned long get_speed(void)
{

    unsigned long adc_value, timer_value, phase_value;

    adc_value = get_ADC();
    adc_value = softswitch(adc_value);
    //Converse adc value into timer reload value
    phase_value = 140 - (adc_value * 3); //adc value to fire phase

    if(DISABLE_HARM_RED_FLAG == FALSE){
		if((phase_value > 40) && (phase_value < 140))
		{
			phase_value = harm_reduce(phase_value);
		}
	}

    timer_value = phase2timer(phase_value);
    adc_value_copied = timer_value;
    return timer_value;
}
//***************************************************************************
//* Name: get_ADC()
//* Input(s) : none.
//* Returns : unsigned int adc_value.
//* Description : get adc value from the I/O port P0.4 and P1.5,
//* if press Key1 only, the adc_value will increas
//* if press Key2 only, the adc_value will decrease
//* if press Key1 and Key2 together, the adc_value will be given
//* with the maxiam value. That will preform softstart demo.
//***************************************************************************
unsigned long get_ADC(void)
{
    unsigned long adc_value;

    if ((key1_flag == 3) && (key2_flag != 3) && (key2_flag != 2))
    {
        if (key_value < speed_max)
        {
            key_value++; //if key1 is pressed, speed increase.
            printf("ADC%u %lu", key_value, adc_value_copied);
            printf(" %u \r\n", DISABLE_HARM_RED_FLAG);
        }
        key1_flag = 0;
    }
    if ((key2_flag == 3) && (key1_flag != 3) && (key1_flag != 2))
    {
        if (key_value > 0x0)
        {
			key_value--; //if key2 is pressed, speed decrease.
			printf("ADC%u %lu", key_value, adc_value_copied);
			printf(" %u \r\n", DISABLE_HARM_RED_FLAG);
        }
        key2_flag = 0;
    }
    if ((key1_flag == 2) && (key2_flag == 2))
    {
//        key_value = speed_max; //if press two keys together, go directly to maximum
       						//speed.
        key_value = 0; //if press two keys together, go directly to maximum
        						//speed.
        key1_flag = 0;
        key2_flag = 0;
    }
    adc_value = key_value;

    return adc_value;

}
//***************************************************************************
//* Name: phase_lookup()
//* Input(s) : unsigned int adc_value.
//* Returns : unsigned int phase_value.
//* Description : convert adc_value into Timer 0 reload phase value
//***************************************************************************
unsigned long harm_reduce(unsigned long phase_value)
{
    unsigned long new_phase_value;
    switch (harm_flag)
    {
    case 0:
            new_phase_value = phase_value - 30;

            break;
    case 1:
            new_phase_value = phase_value - 30;

            break;
    case 2:
            new_phase_value = phase_value + 30;
            break;
    case 3:
            new_phase_value = phase_value + 30;
            break;
    default:
            harm_flag = 0;
            break;
    }
    return new_phase_value;
}
//***************************************************************************
//* Name: softswitch()
//* Input(s) : unsigned int adc_value.
//* Returns : unsigned int adc_new.
//* Description : This method process soft-switch algorithm of the vacuum
//***************************************************************************
unsigned long softswitch(unsigned long adc_value)
{
    unsigned long adc_new;
    if ((adc_old > adc_value) || (adc_value > adc_old) && (update == update_rate))
    {
        if (adc_old > adc_value)
        {
            adc_new = adc_old - one_step;
        }
        else
        {
        adc_new = adc_old + one_step;
        }
        update = 0;
    }
    else
    {
        adc_new = adc_old;
    }
    adc_old = adc_new;
    return adc_new;
}

//***************************************************************************
//* Name: phase2timer()
//* Input(s) : unsigned int phase_value.
//* Returns : unsigned int timer_value.
//* Description : This method process conversion from phase value into timer0
//* reload value
//***************************************************************************
unsigned long phase2timer(unsigned long phase_value)
{
    unsigned long timer_value;
    timer_value = 0xffff - (phase_value * 186);
    return timer_value;
}
